@@ -1,4 +1,12 @@
 class Backend:
+    def __init__(self, name=None, active=True):
+        self.name = name
+        self._active = active
+
+    @property
+    def active(self):
+        return self._active
+
     async def on_start(self, app):
         pass
 
@@ -8,13 +16,13 @@ class Backend:
     def prepare_context(self, ctx):
         pass
 
-    async def perform_updates_request(self, submit_update):
+    async def acquire_updates(self, submit_update):
         raise NotImplementedError
 
-    async def perform_send(self, target_id, message, attachments, kwargs):
+    async def execute_send(self, target_id, message, attachments, kwargs):
         raise NotImplementedError
 
-    async def perform_api_call(self, method, kwargs):
+    async def execute_request(self, method, kwargs):
         raise NotImplementedError
 
     @classmethod
