@@ -4,13 +4,15 @@ from ..update import Message, ReceiverType, UpdateType
 
 
 class Debug(Backend):
-    def __init__(self, messages=None, on_complete=None, save_replies=True):
+    def __init__(self, messages=None, on_complete=None, save_replies=True, **kwargs):
+        super().__init__(**kwargs)
+
         if isinstance(messages, GeneratorType):  # pragma: no cover
             self.messages = messages
-            self.messages_count = -1
         else:
             self.messages = (m for m in messages)
-            self.messages_count = len(messages)
+
+        self.messages_count = 0
 
         self.save_replies = save_replies
         self.answers = {}
@@ -32,18 +34,20 @@ class Debug(Backend):
             date=0,
         )
 
-    async def perform_updates_request(self, submit_update):
+    async def acquire_updates(self, submit_update):
         if not self.messages:
             return
 
         for _ in range(25):
             try:
-                await submit_update(self._make_update(next(self.messages)))
+                message = next(self.messages)
+                self.messages_count += 1
+                await submit_update(self._make_update(message))
             except StopIteration:
                 self.messages = None
                 break
 
-    async def perform_send(self, target_id, message, attachments, kwargs):
+    async def execute_send(self, target_id, message, attachments, kwargs):
         if self.save_replies:
             if target_id not in self.answers:
                 self.answers[target_id] = []
@@ -54,5 +58,11 @@ class Debug(Backend):
 
         self.answers_count += 1
 
-        if self.answers_count == self.messages_count and self.on_complete:
+        self.check_if_complete()
+
+    def check_if_complete(self):
+        if self.messages or not self.on_complete:
+            return
+
+        if self.answers_count == self.messages_count:
             self.on_complete()
