@@ -1,8 +1,11 @@
 import asyncio
 from sortedcontainers import SortedList
 from .handler import HandlerResponse as hr
-from .storages import NaiveMemory
+from .storages import MemoryStorage
+from .storage import OptimisticLockException, Storage
+from .backend import Backend
 from .context import Context
+from .plugin import Plugin
 from .logger import logger
 
 
@@ -10,39 +13,56 @@ class Kutana:
     """
     Main class for kutana application
 
-    :ivar ~.storage: Storage for things like states, e.t.c.
+    Configuration:
+
+    - '.prefixes' - prefixes for commands (default is [".", "/"])
+    - '.ignore_initial_spaces' - ignore spaces after prefix (default is True)
+
     :ivar ~.config: Application's configuration
     """
 
     def __init__(
         self,
-        concurrent_handlers_count=3000,
-        storage=None,
+        concurrent_handlers_count=512,
+        default_storage=None,
         loop=None,
     ):
         self._plugins = []
         self._backends = []
+        self._storages = {"default": default_storage or MemoryStorage()}
 
         self._loop = loop or asyncio.new_event_loop()
 
+        self._concurrent_handlers_count = concurrent_handlers_count
         self._sem = asyncio.Semaphore(
             value=concurrent_handlers_count, loop=self._loop
         )
 
         self._routers = None
-
-        self.storage = storage
+        self._handlers = None
 
         self.config = {
             "prefixes": (".", "/"),
+            "ignore_initial_spaces": True
         }
 
     def get_loop(self):
         """Return application's asyncio loop."""
+        # TODO: Replace with property
         return self._loop
 
+    def set_storage(self, name, storage):
+        if not isinstance(storage, Storage):
+            raise ValueError(f"Provided value is not a storage: {storage}")
+        self._storages[name] = storage
+
+    def get_storage(self, name="default"):
+        return self._storages.get(name)
+
     def add_plugin(self, plugin):
         """Add plugin to the application."""
+        if not isinstance(plugin, Plugin):
+            raise ValueError(f"Provided value is not a plugin: {plugin}")
         if plugin in self._plugins:
             raise RuntimeError("Plugin already added")
         self._plugins.append(plugin)
@@ -58,39 +78,65 @@ class Kutana:
 
     def add_backend(self, backend):
         """Add backend to the application."""
+        if not isinstance(backend, Backend):
+            raise ValueError(f"Provided value is not a backend: {backend}")
         if backend in self._backends:
             raise RuntimeError("Backend already added")
         self._backends.append(backend)
 
+    def get_backend(self, name):
+        """Return first backend with specified name or None."""
+        for backend in self._backends:
+            if backend.name == name:
+                return backend
+        return None
+
     def get_backends(self):
         return self._backends
 
     async def _on_start(self, queue):
+        for storage in self._storages.values():
+            await storage.init()
+
+        # Prepare backends and run background update acquiring
         for backend in self._backends:
             await backend.on_start(self)
 
-            async def perform_updates_request(backend):
-                def submit_update(update):
-                    return queue.put((update, backend))
+            if not backend.active:
+                continue
+
+            async def acquire_updates(backend):  # don't forget to capture backend
+                async def submit_update(update):
+                    return await queue.put((update, backend))
 
                 while True:
-                    await backend.perform_updates_request(submit_update)
+                    if queue.qsize() < queue.maxsize:
+                        await backend.acquire_updates(submit_update)
                     await asyncio.sleep(0)
 
-            asyncio.ensure_future(
-                perform_updates_request(backend),
-                loop=self._loop
-            )
+            asyncio.ensure_future(acquire_updates(backend), loop=self._loop)
 
+        # Prepare plugins
         for plugin in self._plugins:
-            if plugin._on_start:
-                await plugin._on_start(self)
+            plugin.app = self
+
+        # Run event listeners
+        await self._handle_event("start")
+
+    async def _main_loop_wrapper(self):
+        try:
+            await self._main_loop()
+        except asyncio.CancelledError:
+            pass
+        except Exception:
+            self.stop()
+            raise
 
     async def _main_loop(self):
-        if self.storage is None:
-            self.storage = NaiveMemory()
-
-        queue = asyncio.Queue(maxsize=32, loop=self._loop)
+        queue = asyncio.Queue(
+            maxsize=self._concurrent_handlers_count,
+            loop=self._loop
+        )
 
         await self._on_start(queue)
 
@@ -115,6 +161,23 @@ class Kutana:
 
             task.add_done_callback(lambda t: self._sem.release())
 
+    def _init_handlers(self):
+        self._handlers = {}
+
+        for event in ["start", "before", "after", "exception", "shutdown"]:
+            self._handlers[event] = SortedList([], key=lambda r: -r.priority)
+
+        for plugin in self._plugins:
+            for event, handlers in plugin._handlers.items():
+                self._handlers[event].update(handlers)
+
+    async def _handle_event(self, name, *args, **kwargs):
+        if self._handlers is None:
+            self._init_handlers()
+
+        for handler in self._handlers.get(name):
+            await handler.handle(*args, **kwargs)
+
     def _init_routers(self):
         self._routers = SortedList([], key=lambda r: -r.priority)
 
@@ -137,20 +200,17 @@ class Kutana:
             return await self._handle_update(update, ctx)
         except asyncio.CancelledError:
             pass
+        except OptimisticLockException as exc:
+            logger.debug("Optimistic lock exception: %s", exc)
         except Exception as exc:
             logger.exception("Exception while handling the update")
-
-            for plugin in self._plugins:
-                if plugin._on_exception:
-                    await plugin._on_exception(update, ctx, exc)
+            await self._handle_event("exception", update, ctx, exc)
 
     async def _handle_update(self, update, ctx):
         if self._routers is None:
             self._init_routers()
 
-        for plugin in self._plugins:
-            if plugin._on_before:
-                await plugin._on_before(update, ctx)
+        await self._handle_event("before", update, ctx)
 
         for router in self._routers:
             if await router.handle(update, ctx) != hr.SKIPPED:
@@ -159,9 +219,7 @@ class Kutana:
         else:
             ctx._result = hr.SKIPPED
 
-        for plugin in self._plugins:
-            if plugin._on_after:
-                await plugin._on_after(update, ctx, ctx._result)
+        await self._handle_event("after", update, ctx, ctx._result)
 
         return ctx._result
 
@@ -174,9 +232,7 @@ class Kutana:
         for backend in self._backends:
             tasks.append(backend.on_shutdown(self))
 
-        for plugin in self._plugins:
-            if plugin._on_shutdown:
-                tasks.append(plugin._on_shutdown(self))
+        tasks.append(self._handle_event("shutdown"))
 
         await asyncio.gather(*tasks, loop=self._loop, return_exceptions=True)
 
@@ -193,11 +249,11 @@ class Kutana:
         self._loop.stop()
 
     def run(self):
-        """Start the application."""
+        """Run the application."""
         logger.info("Starting application...")
 
         try:
-            asyncio.ensure_future(self._main_loop(), loop=self._loop)
+            asyncio.ensure_future(self._main_loop_wrapper(), loop=self._loop)
             self._loop.run_forever()
         except KeyboardInterrupt:
             asyncio.ensure_future(self._shutdown(), loop=self._loop)
