@@ -1,6 +1,10 @@
 import re
+import inspect
+import warnings
+import functools
 from .handler import Handler, HandlerResponse
 from .update import UpdateType
+from .helpers import ensure_list
 from .backends.vkontakte import VkontaktePluginExtension
 from .routers import (
     CommandsRouter, AttachmentsRouter, ListRouter, AnyMessageRouter,
@@ -16,17 +20,24 @@ class Plugin:
     :param description: Description of the plugin
     """
 
-    def __init__(self, name, description=""):
+    def __init__(self, name, description="", storage="default", **kwargs):
+        self.app = None
         self.name = name
         self.description = description
 
         self._routers = []
+        self._storage = storage
+        self._handlers = {
+            "start": [],
+            "before": [],
+            "after": [],
+            "exception": [],
+            "shutdown": [],
+        }
 
-        self._on_start = None
-        self._on_before = None
-        self._on_after = None
-        self._on_exception = None
-        self._on_shutdown = None
+        # Set provided attributes to this instance
+        for k, v in kwargs.items():
+            setattr(self, k, v)
 
     def _get_or_add_router(self, router_cls, priority=None):
         for router in self._routers:
@@ -41,9 +52,6 @@ class Plugin:
         self._routers.append(router)
         return router
 
-    def _make_handler(self, func, group_state, user_state, priority):
-        return Handler(func, group_state, user_state, priority)
-
     def _add_handler_for_router(self, router, handler, handler_key=None, router_priority=None):
         router = self._get_or_add_router(router, priority=router_priority)
         if handler_key:
@@ -51,67 +59,92 @@ class Plugin:
         else:
             router.add_handler(handler)
 
-    def on_start(self):
+    @property
+    def storage(self):
+        if isinstance(self._storage, str):
+            return self.app.get_storage(self._storage)
+        return self._storage
+
+    def on_start(self, priority=0):
         """
         Decorator for registering coroutine to be called when application
-        starts. It will be passed an application as an argument.
+        starts.
+
+        Priority specifies the order in which handlers are  executed. Handlers
+        with higher priority will be executed first.
         """
         def decorator(func):
-            if self._on_start is not None:
-                raise RuntimeError(f"Hook 'on_start' already set")
-            self._on_start = func
+            async def wrapper():
+                args, *__ = inspect.getfullargspec(func)
+
+                if args:
+                    warnings.warn(
+                        '"on_start" with arguments is deprecated, you can '
+                        'access "app" with "plugin.app"',
+                        DeprecationWarning
+                    )
+                    return await func(self.app)
+                else:
+                    return await func()
+
+            self._handlers["start"].append(Handler(wrapper, priority))
+
             return func
         return decorator
 
-    def on_before(self):
+    def on_before(self, priority=0):
         """
         Decorator for registering coroutine to be called before an update
         will be processed. It will be passed the update and it's context as
         arguments.
+
+        Priority specifies the order in which handlers are executed. Handlers
+        with higher priority will be executed first.
         """
         def decorator(func):
-            if self._on_before is not None:
-                raise RuntimeError(f"Hook 'on_before' already set")
-            self._on_before = func
+            self._handlers["before"].append(Handler(func, priority))
             return func
         return decorator
 
-    def on_after(self):
+    def on_after(self, priority=0):
         """
         Decorator for registering coroutine to be called after an update
         was processed. It will be passed the update, it's context and
         result of processing (:class:`kutana.handler.HandlerResponse`) as
         arguments.
+
+        Priority specifies the order in which handlers are executed. Handlers
+        with higher priority will be executed first.
         """
         def decorator(func):
-            if self._on_after is not None:
-                raise RuntimeError(f"Hook 'on_after' already set")
-            self._on_after = func
+            self._handlers["after"].append(Handler(func, priority))
             return func
         return decorator
 
-    def on_exception(self):
+    def on_exception(self, priority=0):
         """
         Decorator for registering coroutine to be called when exception was
         raised while processing an update. It will be passed the update,
         it's context and raised exception as arguments.
+
+        Priority specifies the order in which handlers are executed. Handlers
+        with higher priority will be executed first.
         """
         def decorator(func):
-            if self._on_exception is not None:
-                raise RuntimeError(f"Hook 'on_exception' already set")
-            self._on_exception = func
+            self._handlers["exception"].append(Handler(func, priority))
             return func
         return decorator
 
-    def on_shutdown(self):
+    def on_shutdown(self, priority=0):
         """
         Decorator for registering coroutine to be called when application
         will be stopped. It will be passed application as an argument.
+
+        Priority specifies the order in which handlers are executed. Handlers
+        with higher priority will be executed first.
         """
         def decorator(func):
-            if self._on_shutdown is not None:
-                raise RuntimeError(f"Hook 'on_shutdown' already set")
-            self._on_shutdown = func
+            self._handlers["shutdown"].append(Handler(func, priority))
             return func
         return decorator
 
@@ -123,8 +156,6 @@ class Plugin:
     def on_commands(
         self,
         commands,
-        group_state="*",
-        user_state="*",
         priority=0,
         router_priority=None
     ):
@@ -143,10 +174,6 @@ class Plugin:
         - body
         - match
 
-        You can specify group_state and/or user_state to control
-        which state must have groups and users attempting to
-        access this coroutine.
-
         Priority is a value that used to determine order in which
         coroutines are executed. Router priority is a value that
         used to determine the order in which routers is checked
@@ -159,14 +186,13 @@ class Plugin:
         coroutines are not executed further.
         """
 
-        if not isinstance(commands, (list, tuple)):
-            raise ValueError("`commands` argument must be list or tuple")
+        commands = ensure_list(commands)
 
         def decorator(func):
             for command in commands:
                 self._add_handler_for_router(
                     CommandsRouter,
-                    handler=self._make_handler(func, group_state, user_state, priority),
+                    handler=Handler(func, priority),
                     handler_key=command,
                     router_priority=router_priority,
                 )
@@ -177,8 +203,6 @@ class Plugin:
     def on_attachments(
         self,
         types,
-        group_state="*",
-        user_state="*",
         priority=0,
         router_priority=None
     ):
@@ -188,14 +212,16 @@ class Plugin:
         attachment with one of specified types.
 
         See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
+        about 'priority' and 'router_priority'.
         """
 
+        types = ensure_list(types)
+
         def decorator(func):
             for atype in types:
                 self._add_handler_for_router(
                     AttachmentsRouter,
-                    handler=self._make_handler(func, group_state, user_state, priority),
+                    handler=Handler(func, priority),
                     handler_key=atype,
                     router_priority=router_priority,
                 )
@@ -203,31 +229,46 @@ class Plugin:
 
         return decorator
 
-    def on_any_message(self, group_state="*", user_state="*", priority=0, router_priority=None):
-        """
-        Decorator for registering coroutine to be called when
-        incoming update is message. This will always be called.
-        If you want to catch all unprocessed message, you should
-        use :meth:`kutana.plugin.Plugin.on_any_unprocessed_message`.
-
-        See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
-        """
+    def on_any_message(self, *args, **kwargs):
+        warnings.warn(
+            '"on_any_message" is deprecated, use "on_messages" instead',
+            DeprecationWarning
+        )
+        return self.on_messages(*args, **kwargs)
 
+    def _make_decorator(self, router, priority, router_priority):
         def decorator(func):
             self._add_handler_for_router(
-                AnyMessageRouter,
-                handler=self._make_handler(func, group_state, user_state, priority),
+                router,
+                handler=Handler(func, priority),
                 router_priority=router_priority,
             )
             return func
 
         return decorator
 
-    def on_any_unprocessed_message(
+    def on_messages(self, priority=0, router_priority=None):
+        """
+        Decorator for registering coroutine to be called when
+        incoming update is message. This will always be called.
+        If you want to catch all unprocessed message, you should
+        use :meth:`kutana.plugin.Plugin.on_unprocessed_messages`.
+
+        See :class:`kutana.plugin.Plugin.on_commands` for details
+        about 'priority' and 'router_priority'.
+        """
+
+        return self._make_decorator(AnyMessageRouter, priority, router_priority)
+
+    def on_any_unprocessed_message(self, *args, **kwargs):
+        warnings.warn(
+            '"on_any_unprocessed_message" is deprecated, use "on_unprocessed_messages" instead',
+            DeprecationWarning
+        )
+        return self.on_unprocessed_messages(*args, **kwargs)
+
+    def on_unprocessed_messages(
         self,
-        group_state="*",
-        user_state="*",
         priority=0,
         router_priority=-3
     ):
@@ -237,20 +278,12 @@ class Plugin:
         coroutine processed update.
 
         See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
+        about 'priority' and 'router_priority'.
         """
 
-        def decorator(func):
-            self._add_handler_for_router(
-                AnyMessageRouter,
-                handler=self._make_handler(func, group_state, user_state, priority),
-                router_priority=router_priority,
-            )
-            return func
+        return self._make_decorator(AnyMessageRouter, priority, router_priority)
 
-        return decorator
-
-    def on_match(self, pattern, group_state="*", user_state="*", priority=0, router_priority=-3):
+    def on_match(self, pattern, priority=0, router_priority=-3):
         """
         Decorator for registering coroutine to be called when
         incoming update is message and it's text is matched by
@@ -261,7 +294,7 @@ class Plugin:
         - match
 
         See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
+        about 'priority' and 'router_priority'.
         """
 
         def decorator(func):
@@ -279,7 +312,7 @@ class Plugin:
 
             self._add_handler_for_router(
                 ListRouter,
-                handler=self._make_handler(wrapper, group_state, user_state, priority),
+                handler=Handler(wrapper, priority),
                 router_priority=router_priority,
             )
 
@@ -287,43 +320,115 @@ class Plugin:
 
         return decorator
 
-    def on_any_update(self, group_state="*", user_state="*", priority=0, router_priority=None):
+    def on_any_update(self, *args, **kwargs):
+        warnings.warn(
+            '"on_any_update" is deprecated, use "on_updates" instead',
+            DeprecationWarning
+        )
+        return self.on_updates(self, *args, **kwargs)
+
+    def on_updates(self, priority=0, router_priority=None):
         """
         Decorator for registering coroutine to be called when
         incoming update is not message. This will always be called.
         If you want to catch all unprocessed message, you should
-        use :meth:`kutana.plugin.Plugin.on_any_unprocessed_update`.
+        use :meth:`kutana.plugin.Plugin.on_unprocessed_updates`.
 
         See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
+        about 'priority' and 'router_priority'.
         """
 
-        def decorator(func):
-            self._add_handler_for_router(
-                AnyUpdateRouter,
-                handler=self._make_handler(func, group_state, user_state, priority),
-                router_priority=router_priority,
-            )
-            return func
+        return self._make_decorator(AnyUpdateRouter, priority, router_priority)
 
-        return decorator
+    def on_any_unprocessed_update(self, *args, **kwargs):
+        warnings.warn(
+            '"on_any_unprocessed_update" is deprecated, use "on_unprocessed_updates" instead',
+            DeprecationWarning
+        )
+        return self.on_unprocessed_updates(self, *args, **kwargs)
 
-    def on_any_unprocessed_update(self, group_state="*", user_state="*", priority=0, router_priority=-3):
+    def on_unprocessed_updates(self, priority=0, router_priority=-3):
         """
         Decorator for registering coroutine to be called when
         incoming update is update. This will be called if no other
         coroutine processed update.
 
         See :class:`kutana.plugin.Plugin.on_commands` for details
-        about 'group_state', 'user_state', 'priority' and 'router_priority'.
+        about 'priority' and 'router_priority'.
+        """
+
+        return self._make_decorator(AnyUpdateRouter, priority, router_priority)
+
+    def expect_sender(self, state=None, localized=False):
+        """
+        This decorator does following things:
+
+        - It adds 'sender' to the context. Object that contains data you
+            previously saved to it. You can treat it as object or dict
+            with data. In order to save your changes, you should await
+            'save' method (or use 'update').
+
+        - If you specified 'state', this will skip all users that don't have
+            specified value in 'state' field.
+
+        - If you 'localized' is true, state will be limited only to current
+            receiver.
+
+        NOTE:
+            If you will attempt to update state, but between reading and
+            writing (updating) someone already updated it 'OptimisticLockException'
+            will be raised. By default this exceptions are ignored!
         """
 
         def decorator(func):
-            self._add_handler_for_router(
-                AnyUpdateRouter,
-                handler=self._make_handler(func, group_state, user_state, priority),
-                router_priority=router_priority,
-            )
-            return func
+            @functools.wraps(func)
+            async def wrapper(upd, ctx, *args, **kwargs):
+                sender = await self.storage.get(ctx.sender_key)
 
+                if not sender or "state" not in sender:
+                    sender = self.storage.make_document(
+                        {"state": "", "_version": None}, ctx.sender_key
+                    )
+
+                if state is not None and sender["state"] != state:
+                    return HandlerResponse.SKIPPED
+
+                ctx.sender = sender
+
+                return await func(upd, ctx, *args, **kwargs)
+            return wrapper
+        return decorator
+
+    def expect_receiver(self, state=None):
+        """
+        This decorator does following things:
+
+        - It adds 'receiver' to the context. Object that contains data you
+            previously saved to it. You can treat it as object or dict
+            with data. In order to save your changes, you should await
+            'save' method (or use 'update').
+
+        NOTE:
+            If you will attempt to update state, but between reading and
+            writing (updating) someone already updated it - 'OptimisticLockException'
+            will be raised. By default this exceptions are ignored!
+        """
+
+        def decorator(func):
+            @functools.wraps(func)
+            async def wrapper(upd, ctx, *args, **kwargs):
+                receiver = await self.storage.get(ctx.receiver_key)
+
+                if not receiver or "state" not in receiver:
+                    receiver = self.storage.make_document(
+                        {"state": "", "_version": None}, ctx.receiver_key
+                    )
+
+                if state is not None and receiver["state"] != state:
+                    return HandlerResponse.SKIPPED
+
+                ctx.receiver = receiver
+
+                return await func(upd, ctx, *args, **kwargs)
+            return wrapper
         return decorator
